/* Copyright (c) 1988 The Regents of the University of California. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in all
 * advertising materials mentioning features or use of this software. Neither
 * the name of the University nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission. THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 *
 * @(#)utmp.h   5.10 (Berkeley) 6/1/90 */

#define _PATH_UTMP  "/etc/utmp"
#define _PATH_WTMP  "/usr/adm/wtmp"
#define _PATH_LASTLOG   "/usr/adm/lastlog"

#define UT_NAMESIZE 8
#define UT_LINESIZE 8
#define UT_HOSTSIZE 16

struct lastlog {
    time_t ll_time;
    char ll_line[UT_LINESIZE];
    char ll_host[UT_HOSTSIZE];
};

struct utmp {
    char ut_line[UT_LINESIZE];
    char ut_name[UT_NAMESIZE];
    char ut_host[UT_HOSTSIZE];
    long ut_time;
};
